@@ -446,10 +446,6 @@ dblink_fetch(PG_FUNCTION_ARGS)
 		/* got results, keep track of them */
 		funcctx->user_fctx = res;
 
-		/* fast track when no results */
-		if (funcctx->max_calls < 1)
-			SRF_RETURN_DONE(funcctx);
-
 		/* check typtype to see if we have a predetermined return type */
 		functypeid = get_func_rettype(funcid);
 		functyptype = get_typtype(functypeid);
@@ -474,6 +470,22 @@ dblink_fetch(PG_FUNCTION_ARGS)
 		/* store needed metadata for subsequent calls */
 		slot = TupleDescGetSlot(tupdesc);
 		funcctx->slot = slot;
+
+		/* check result and tuple descriptor have the same number of columns */
+		if (PQnfields(res) != tupdesc->natts)
+			ereport(ERROR,
+					(errcode(ERRCODE_DATATYPE_MISMATCH),
+				errmsg("remote query result rowtype does not match "
+						"the specified FROM clause rowtype")));
+
+		/* fast track when no results */
+		if (funcctx->max_calls < 1)
+		{
+			if (res)
+				PQclear(res);
+			SRF_RETURN_DONE(funcctx);
+		}
+
 		attinmeta = TupleDescGetAttInMetadata(tupdesc);
 		funcctx->attinmeta = attinmeta;
 
@@ -617,10 +629,6 @@ dblink_record(PG_FUNCTION_ARGS)
 		if (freeconn && PG_NARGS() == 2)
 			PQfinish(conn);
 
-		/* fast track when no results */
-		if (funcctx->max_calls < 1)
-			SRF_RETURN_DONE(funcctx);
-
 		/* check typtype to see if we have a predetermined return type */
 		functypeid = get_func_rettype(funcid);
 		functyptype = get_typtype(functypeid);
@@ -648,6 +656,22 @@ dblink_record(PG_FUNCTION_ARGS)
 		/* store needed metadata for subsequent calls */
 		slot = TupleDescGetSlot(tupdesc);
 		funcctx->slot = slot;
+
+		/* check result and tuple descriptor have the same number of columns */
+		if (PQnfields(res) != tupdesc->natts)
+			ereport(ERROR,
+					(errcode(ERRCODE_DATATYPE_MISMATCH),
+				errmsg("remote query result rowtype does not match "
+						"the specified FROM clause rowtype")));
+
+		/* fast track when no results */
+		if (funcctx->max_calls < 1)
+		{
+			if (res)
+				PQclear(res);
+			SRF_RETURN_DONE(funcctx);
+		}
+
 		attinmeta = TupleDescGetAttInMetadata(tupdesc);
 		funcctx->attinmeta = attinmeta;
 
