@@ -16,7 +16,7 @@
  *
  *
  * IDENTIFICATION
- *		$Header: /cvsroot/pgsql/src/bin/pg_dump/pg_backup_tar.c,v 1.32.2.2 2003/02/01 19:29:26 tgl Exp $
+ *		$Header: /cvsroot/pgsql/src/bin/pg_dump/pg_backup_tar.c,v 1.38.2.1 2005/06/22 02:12:19 neilc Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -158,10 +158,11 @@ InitArchiveFmt_Tar(ArchiveHandle *AH)
 	/*
 	 * Set up some special context used in compressing data.
 	 */
-	ctx = (lclContext *) malloc(sizeof(lclContext));
+	ctx = (lclContext *) calloc(1, sizeof(lclContext));
 	AH->formatData = (void *) ctx;
 	ctx->filePos = 0;
-
+	ctx->isSpecialScript = 0;
+	
 	/* Initialize LO buffering */
 	AH->lo_buf_size = LOBBUFSIZE;
 	AH->lo_buf = (void *) malloc(LOBBUFSIZE);
@@ -253,7 +254,7 @@ _ArchiveEntry(ArchiveHandle *AH, TocEntry *te)
 	lclTocEntry *ctx;
 	char		fn[K_STD_BUF_SIZE];
 
-	ctx = (lclTocEntry *) malloc(sizeof(lclTocEntry));
+	ctx = (lclTocEntry *) calloc(1, sizeof(lclTocEntry));
 	if (te->dataDumper != NULL)
 	{
 #ifdef HAVE_LIBZ
@@ -292,7 +293,7 @@ _ReadExtraToc(ArchiveHandle *AH, TocEntry *te)
 
 	if (ctx == NULL)
 	{
-		ctx = (lclTocEntry *) malloc(sizeof(lclTocEntry));
+		ctx = (lclTocEntry *) calloc(1, sizeof(lclTocEntry));
 		te->formatData = (void *) ctx;
 	}
 
@@ -374,7 +375,7 @@ tarOpen(ArchiveHandle *AH, const char *filename, char mode)
 			sprintf(fmode, "wb%d", AH->compression);
 			tm->zFH = gzdopen(dup(fileno(tm->tmpFH)), fmode);
 			if (tm->zFH == NULL)
-				die_horribly(AH, modulename, "could not gzdopen temporary file\n");
+				die_horribly(AH, modulename, "could not open temporary file\n");
 
 		}
 		else
@@ -504,7 +505,7 @@ _tarReadRaw(ArchiveHandle *AH, void *buf, size_t len, TAR_MEMBER *th, FILE *fh)
 				res = fread(&((char *) buf)[used], 1, len, th->nFH);
 		}
 		else
-			die_horribly(AH, modulename, "neither th nor fh specified in tarReadRaw() (internal error)\n");
+			die_horribly(AH, modulename, "internal error -- neither th nor fh specified in tarReadRaw()\n");
 	}
 
 #if 0
@@ -636,7 +637,7 @@ _PrintTocData(ArchiveHandle *AH, TocEntry *te, RestoreOptions *ropt)
 		 */
 		if (strncmp(tmpCopy, "copy ", 5) != 0)
 			die_horribly(AH, modulename,
-						 "bad COPY statement - could not find \"copy\" in string \"%s\"\n", tmpCopy);
+						 "invalid COPY statement -- could not find \"copy\" in string \"%s\"\n", tmpCopy);
 
 		pos1 = 5;
 		for (pos1 = 5; pos1 < strlen(tmpCopy); pos1++)
@@ -654,7 +655,7 @@ _PrintTocData(ArchiveHandle *AH, TocEntry *te, RestoreOptions *ropt)
 
 		if (pos2 >= strlen(tmpCopy))
 			die_horribly(AH, modulename,
-						 "bad COPY statement - could not find \"from stdin\" in string \"%s\" starting at position %lu\n",
+						 "invalid COPY statement -- could not find \"from stdin\" in string \"%s\" starting at position %lu\n",
 						 tmpCopy, (unsigned long) pos1);
 
 		ahwrite(tmpCopy, 1, pos2, AH);	/* 'copy "table" [with oids]' */
@@ -1044,8 +1045,8 @@ _tarAddFile(ArchiveHandle *AH, TAR_MEMBER *th)
 		char		buf1[100],
 					buf2[100];
 
-		snprintf(buf1, 100, INT64_FORMAT, (int64) len);
-		snprintf(buf2, 100, INT64_FORMAT, (int64) th->pos);
+		snprintf(buf1, sizeof(buf1), INT64_FORMAT, (int64) len);
+		snprintf(buf2, sizeof(buf2), INT64_FORMAT, (int64) th->pos);
 		die_horribly(AH, modulename, "actual file length (%s) does not match expected (%s)\n",
 					 buf1, buf2);
 	}
@@ -1081,8 +1082,8 @@ _tarPositionTo(ArchiveHandle *AH, const char *filename)
 		char		buf1[100],
 					buf2[100];
 
-		snprintf(buf1, 100, INT64_FORMAT, (int64) ctx->tarFHpos);
-		snprintf(buf2, 100, INT64_FORMAT, (int64) ctx->tarNextMember);
+		snprintf(buf1, sizeof(buf1), INT64_FORMAT, (int64) ctx->tarFHpos);
+		snprintf(buf2, sizeof(buf2), INT64_FORMAT, (int64) ctx->tarNextMember);
 		ahlog(AH, 4, "moving from position %s to next member at file position %s\n",
 			  buf1, buf2);
 
@@ -1093,7 +1094,7 @@ _tarPositionTo(ArchiveHandle *AH, const char *filename)
 	{
 		char		buf[100];
 
-		snprintf(buf, 100, INT64_FORMAT, (int64) ctx->tarFHpos);
+		snprintf(buf, sizeof(buf), INT64_FORMAT, (int64) ctx->tarFHpos);
 		ahlog(AH, 4, "now at file position %s\n", buf);
 	}
 
@@ -1150,9 +1151,8 @@ _tarGetHeader(ArchiveHandle *AH, TAR_MEMBER *th)
 	int			sum,
 				chk;
 	size_t		len;
-	unsigned long	ullen;
+	unsigned long ullen;
 	off_t		hPos;
-	int			i;
 	bool		gotBlock = false;
 
 	while (!gotBlock)
@@ -1163,8 +1163,8 @@ _tarGetHeader(ArchiveHandle *AH, TAR_MEMBER *th)
 			char		buf1[100],
 						buf2[100];
 
-			snprintf(buf1, 100, INT64_FORMAT, (int64) ftello(ctx->tarFH));
-			snprintf(buf2, 100, INT64_FORMAT, (int64) ftello(ctx->tarFHpos));
+			snprintf(buf1, sizeof(buf1), INT64_FORMAT, (int64) ftello(ctx->tarFH));
+			snprintf(buf2, sizeof(buf2), INT64_FORMAT, (int64) ftello(ctx->tarFHpos));
 			die_horribly(AH, modulename,
 						 "mismatch in actual vs. predicted file position (%s vs. %s)\n",
 						 buf1, buf2);
@@ -1175,7 +1175,7 @@ _tarGetHeader(ArchiveHandle *AH, TAR_MEMBER *th)
 		hPos = ctx->tarFHpos;
 
 		/* Read a 512 byte block, return EOF, exit if short */
-		len = _tarReadRaw(AH, &h[0], 512, NULL, ctx->tarFH);
+		len = _tarReadRaw(AH, h, 512, NULL, ctx->tarFH);
 		if (len == 0)			/* EOF */
 			return 0;
 
@@ -1185,20 +1185,22 @@ _tarGetHeader(ArchiveHandle *AH, TAR_MEMBER *th)
 						 (unsigned long) len);
 
 		/* Calc checksum */
-		chk = _tarChecksum(&h[0]);
+		chk = _tarChecksum(h);
+		sscanf(&h[148], "%8o", &sum);
 
 		/*
-		 * If the checksum failed, see if it is a null block. If so, then
-		 * just try with next block...
+		 * If the checksum failed, see if it is a null block. If so,
+		 * silently continue to the next block.
 		 */
-
 		if (chk == sum)
 			gotBlock = true;
 		else
 		{
+			int			i;
+
 			for (i = 0; i < 512; i++)
 			{
-				if (h[0] != 0)
+				if (h[i] != 0)
 				{
 					gotBlock = true;
 					break;
@@ -1210,12 +1212,11 @@ _tarGetHeader(ArchiveHandle *AH, TAR_MEMBER *th)
 	sscanf(&h[0], "%99s", tag);
 	sscanf(&h[124], "%12lo", &ullen);
 	len = (size_t) ullen;
-	sscanf(&h[148], "%8o", &sum);
 
 	{
 		char		buf[100];
 
-		snprintf(buf, 100, INT64_FORMAT, (int64) hPos);
+		snprintf(buf, sizeof(buf), INT64_FORMAT, (int64) hPos);
 		ahlog(AH, 3, "TOC Entry %s at %s (length %lu, checksum %d)\n",
 			  tag, buf, (unsigned long) len, sum);
 	}
@@ -1224,7 +1225,7 @@ _tarGetHeader(ArchiveHandle *AH, TAR_MEMBER *th)
 	{
 		char		buf[100];
 
-		snprintf(buf, 100, INT64_FORMAT, (int64) ftello(ctx->tarFH));
+		snprintf(buf, sizeof(buf), INT64_FORMAT, (int64) ftello(ctx->tarFH));
 		die_horribly(AH, modulename,
 					 "corrupt tar header found in %s "
 					 "(expected %d, computed %d) file position %s\n",
@@ -1242,15 +1243,18 @@ _tarGetHeader(ArchiveHandle *AH, TAR_MEMBER *th)
  * Utility routine to print possibly larger than 32 bit integers in a
  * portable fashion.  Filled with zeros.
  */
-static void print_val(char *s, uint64 val, unsigned int base, size_t len)
+static void
+print_val(char *s, uint64 val, unsigned int base, size_t len)
 {
-        int i;
-        for (i = len; i > 0; i--)
-        {
-                int digit = val % base;
-                s[i - 1] = '0' + digit;
-                val = val / base;
-        }
+	int			i;
+
+	for (i = len; i > 0; i--)
+	{
+		int			digit = val % base;
+
+		s[i - 1] = '0' + digit;
+		val = val / base;
+	}
 }
 
 
@@ -1316,6 +1320,6 @@ _tarWriteHeader(TAR_MEMBER *th)
 	}
 
 	if (fwrite(h, 1, 512, th->tarFH) != 512)
-		die_horribly(th->AH, modulename, "unable to write tar header\n");
+		die_horribly(th->AH, modulename, "could not write tar header\n");
 
 }
