@@ -9,10 +9,10 @@
  *	  polluting the namespace with lots of stuff...
  *
  *
- * Portions Copyright (c) 1996-2002, PostgreSQL Global Development Group
+ * Portions Copyright (c) 1996-2003, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
- * $Id: c.h,v 1.130.2.1 2005/07/18 15:54:30 tgl Exp $
+ * $Id: c.h,v 1.156.2.1 2005/07/18 15:54:11 tgl Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -51,6 +51,10 @@
  */
 
 #include "pg_config.h"
+#include "pg_config_manual.h"	/* must be after pg_config.h */
+#ifndef WIN32
+#include "pg_config_os.h"		/* must be before any system header files */
+#endif
 #include "postgres_ext.h"
 
 #include <stdio.h>
@@ -63,14 +67,19 @@
 #endif
 #include <sys/types.h>
 
-#ifdef __CYGWIN__
 #include <errno.h>
-#include <sys/fcntl.h>			/* ensure O_BINARY is available */
+#if defined(__CYGWIN__) || defined(WIN32)
+#include <fcntl.h>				/* ensure O_BINARY is available */
 #endif
 #ifdef HAVE_SUPPORTDEFS_H
 #include <SupportDefs.h>
 #endif
 
+#if defined(WIN32) && !defined(_MSC_VER) && !defined(__BORLANDC__)
+/* We have to redefine some system functions after they are included above */
+#include "pg_config_os.h"
+#endif
+
 /* Must be before gettext() games below */
 #include <locale.h>
 
@@ -319,6 +328,14 @@ typedef unsigned long int uint64;
 #define HAVE_INT64_TIMESTAMP
 #endif
 
+/* Global variable holding time zone information. */
+#ifndef HAVE_UNDERSCORE_TIMEZONE
+#define TIMEZONE_GLOBAL timezone
+#else
+#define TIMEZONE_GLOBAL _timezone
+#define tzname _tzname			/* should be in time.h? */
+#endif
+
 /* sig_atomic_t is required by ANSI C, but may be missing on old platforms */
 #ifndef HAVE_SIG_ATOMIC_T
 typedef int sig_atomic_t;
@@ -358,7 +375,7 @@ typedef float float4;
 typedef double float8;
 
 /*
- * Oid, RegProcedure, TransactionId, CommandId
+ * Oid, RegProcedure, TransactionId, CommandId, AclId
  */
 
 /* typedef Oid is in postgres_ext.h */
@@ -376,6 +393,8 @@ typedef uint32 CommandId;
 
 #define FirstCommandId	((CommandId) 0)
 
+typedef int32 AclId;			/* user and group identifiers */
+
 /*
  * Array indexing support
  */
@@ -462,6 +481,8 @@ typedef NameData *Name;
 
 #define OidIsValid(objectId)  ((bool) ((objectId) != InvalidOid))
 
+#define AclIdIsValid(aclId)  ((bool) ((aclId) != 0))
+
 #define RegProcedureIsValid(p)	OidIsValid(p)
 
 
@@ -503,13 +524,16 @@ typedef NameData *Name;
  * ----------------
  */
 
-#define TYPEALIGN(ALIGNVAL,LEN) (((long)(LEN) + (ALIGNVAL-1)) & ~(ALIGNVAL-1))
+#define TYPEALIGN(ALIGNVAL,LEN)  \
+	(((long) (LEN) + (ALIGNVAL-1)) & ~((long) (ALIGNVAL-1)))
 
 #define SHORTALIGN(LEN)			TYPEALIGN(ALIGNOF_SHORT, (LEN))
 #define INTALIGN(LEN)			TYPEALIGN(ALIGNOF_INT, (LEN))
 #define LONGALIGN(LEN)			TYPEALIGN(ALIGNOF_LONG, (LEN))
 #define DOUBLEALIGN(LEN)		TYPEALIGN(ALIGNOF_DOUBLE, (LEN))
 #define MAXALIGN(LEN)			TYPEALIGN(MAXIMUM_ALIGNOF, (LEN))
+/* MAXALIGN covers only built-in types, not buffers */
+#define BUFFERALIGN(LEN)		TYPEALIGN(ALIGNOF_BUFFER, (LEN))
 
 
 /* ----------------------------------------------------------------
@@ -572,7 +596,7 @@ typedef NameData *Name;
  *	Exactly the same as standard library function memset(), but considerably
  *	faster for zeroing small word-aligned structures (such as parsetree nodes).
  *	This has to be a macro because the main point is to avoid function-call
- *	overhead.   However, we have also found that the loop is faster than
+ *	overhead.	However, we have also found that the loop is faster than
  *	native libc memset() on some platforms, even those with assembler
  *	memset() functions.  More research needs to be done, perhaps with
  *	platform-specific MEMSET_LOOP_LIMIT values or tests in configure.
@@ -602,6 +626,55 @@ typedef NameData *Name;
 
 #define MEMSET_LOOP_LIMIT  1024
 
+/*
+ * MemSetAligned is the same as MemSet except it omits the test to see if
+ * "start" is word-aligned.  This is okay to use if the caller knows a-priori
+ * that the pointer is suitably aligned (typically, because he just got it
+ * from palloc(), which always delivers a max-aligned pointer).
+ */
+#define MemSetAligned(start, val, len) \
+	do \
+	{ \
+		int32 * _start = (int32 *) (start); \
+		int		_val = (val); \
+		Size	_len = (len); \
+\
+		if ((_len & INT_ALIGN_MASK) == 0 && \
+			_val == 0 && \
+			_len <= MEMSET_LOOP_LIMIT) \
+		{ \
+			int32 * _stop = (int32 *) ((char *) _start + _len); \
+			while (_start < _stop) \
+				*_start++ = 0; \
+		} \
+		else \
+			memset((char *) _start, _val, _len); \
+	} while (0)
+
+
+/*
+ * MemSetTest/MemSetLoop are a variant version that allow all the tests in
+ * MemSet to be done at compile time in cases where "val" and "len" are
+ * constants *and* we know the "start" pointer must be word-aligned.
+ * If MemSetTest succeeds, then it is okay to use MemSetLoop, otherwise use
+ * MemSetAligned.  Beware of multiple evaluations of the arguments when using
+ * this approach.
+ */
+#define MemSetTest(val, len) \
+	( ((len) & INT_ALIGN_MASK) == 0 && \
+	(len) <= MEMSET_LOOP_LIMIT && \
+	(val) == 0 )
+
+#define MemSetLoop(start, val, len) \
+	do \
+	{ \
+		int32 * _start = (int32 *) (start); \
+		int32 * _stop = (int32 *) ((char *) _start + (Size) (len)); \
+	\
+		while (_start < _stop) \
+			*_start++ = 0; \
+	} while (0)
+
 
 /* ----------------------------------------------------------------
  *				Section 7:	random stuff
@@ -626,7 +699,7 @@ typedef NameData *Name;
  * ----------------------------------------------------------------
  */
 
-#ifdef __CYGWIN__
+#if defined(__CYGWIN__) || defined(WIN32)
 #define PG_BINARY	O_BINARY
 #define PG_BINARY_R "rb"
 #define PG_BINARY_W "wb"
@@ -636,13 +709,22 @@ typedef NameData *Name;
 #define PG_BINARY_W "w"
 #endif
 
-#if defined(sun) && defined(__sparc__) && !defined(__SVR4)
-#include <unistd.h>
+#if !defined(WIN32) && !defined(__BEOS__)
+#define FCNTL_NONBLOCK(sock)	fcntl(sock, F_SETFL, O_NONBLOCK)
+#else
+extern long ioctlsocket_ret;
+
+/* Returns non-0 on failure, while fcntl() returns -1 on failure */
+#ifdef WIN32
+#define FCNTL_NONBLOCK(sock)	((ioctlsocket(sock, FIONBIO, &ioctlsocket_ret) == 0) ? 0 : -1)
+#endif
+#ifdef __BEOS__
+#define FCNTL_NONBLOCK(sock)	((ioctl(sock, FIONBIO, &ioctlsocket_ret) == 0) ? 0 : -1)
+#endif
 #endif
 
-#if defined(bsdi) || defined(netbsd)
-int fseeko(FILE *stream, off_t offset, int whence);
-off_t ftello(FILE *stream);
+#if defined(sun) && defined(__sparc__) && !defined(__SVR4)
+#include <unistd.h>
 #endif
 
 /* These are for things that are one way on Unix and another on NT */
@@ -650,8 +732,7 @@ off_t ftello(FILE *stream);
 
 /*
  * Provide prototypes for routines not present in a particular machine's
- * standard C library.	It'd be better to put these in pg_config.h, but
- * in pg_config.h we haven't yet included anything that defines size_t...
+ * standard C library.
  */
 
 #if !HAVE_DECL_SNPRINTF
@@ -669,4 +750,51 @@ extern int	vsnprintf(char *str, size_t count, const char *fmt, va_list args);
 #define memmove(d, s, c)		bcopy(s, d, c)
 #endif
 
+#ifndef DLLIMPORT
+#define DLLIMPORT				/* no special DLL markers on most ports */
+#endif
+
+/*
+ * The following is used as the arg list for signal handlers.  Any ports
+ * that take something other than an int argument should override this in
+ * their pg_config_os.h file.  Note that variable names are required
+ * because it is used in both the prototypes as well as the definitions.
+ * Note also the long name.  We expect that this won't collide with
+ * other names causing compiler warnings.
+ */
+
+#ifndef SIGNAL_ARGS
+#define SIGNAL_ARGS  int postgres_signal_arg
+#endif
+
+/*
+ * When there is no sigsetjmp, its functionality is provided by plain
+ * setjmp. Incidentally, nothing provides setjmp's functionality in
+ * that case.
+ */
+#ifndef HAVE_SIGSETJMP
+#define sigjmp_buf jmp_buf
+#define sigsetjmp(x,y) setjmp(x)
+#define siglongjmp longjmp
+#endif
+
+#if defined(HAVE_FDATASYNC) && !HAVE_DECL_FDATASYNC
+extern int	fdatasync(int fildes);
+#endif
+
+/* If strtoq() exists, rename it to the more standard strtoll() */
+#if defined(HAVE_LONG_LONG_INT_64) && !defined(HAVE_STRTOLL) && defined(HAVE_STRTOQ)
+#define strtoll strtoq
+#define HAVE_STRTOLL 1
+#endif
+
+/* If strtouq() exists, rename it to the more standard strtoull() */
+#if defined(HAVE_LONG_LONG_INT_64) && !defined(HAVE_STRTOULL) && defined(HAVE_STRTOUQ)
+#define strtoull strtouq
+#define HAVE_STRTOULL 1
+#endif
+
+/* /port compatibility functions */
+#include "port.h"
+
 #endif   /* C_H */
